-- Author: Paolo Camurati
-- File : Decimal_count_beh.vhd
-- Design units:
-- entity Decimal_count
-- function: 3-digit Decimal counter 
-- with asynchronous reset and enable 
-- input: Clk, Rst, En (1 bit)
-- output: output (3 x 4 bits)
-- architecture Decimal_count_beh
-- Library/package:ieee.std_logic_ll64, ieee.numeric_std: to
-- use std-logic and numeric
-- Synthesis and verification (optional):
-- Synthesis software: . . .
--  Options/script: . . .
--  Target technology: . . .
--  Testbench: Decimal_count_tb.vhd
--  Revision history
--  Version 1.0
--  Date: 20170614
--  Comments:

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY Decimal_count IS
  PORT (Clk, Rst, En: IN std_logic; 
        out2, out1, out0: OUT std_logic_vector(3 DOWNTO 0));
END Decimal_count;

ARCHITECTURE Decimal_count_beh OF Decimal_count IS
  SIGNAL d100Curr, d100Next, d10Curr, d10Next, d1Curr, d1Next: unsigned(3 DOWNTO 0);

BEGIN

  StateReg: PROCESS (Clk, Rst, En)
  BEGIN
    IF (Rst = '1') THEN
      d100Curr <= (OTHERS =>'0');
      d10Curr  <= (OTHERS =>'0');
      d1Curr   <= (OTHERS =>'0');
    ELSIF (En = '1') THEN
      IF (Clk = '1' AND Clk'EVENT) THEN
        d100Curr <= d100Next;
        d10Curr  <= d10Next;
        d1Curr   <= d1Next;
      END IF;
    END IF;
  END PROCESS;

  d100Next <= "0000"     WHEN (d100Curr=9 AND d10Curr=9 AND d1Curr=9) ELSE
              d100Curr+1 WHEN (d10Curr=9 AND d1Curr=9) ELSE
              d100Curr; 
  d10Next  <= "0000"     WHEN (d10Curr=9 AND d1Curr=9) ELSE
              d10Curr+1  WHEN (d1Curr=9) ELSE
              d10Curr;
  d1Next   <= "0000"     WHEN (d1Curr=9) ELSE
              d1Curr+1;
				  
  out2 <= std_logic_vector(d100Curr);
  out1  <= std_logic_vector(d10Curr);
  out0   <= std_logic_vector(d1Curr);
END Decimal_count_beh;
